/* START LIBRARY DESCRIPTION ************************************
FILE:				DiffCon Command.c
PARENT FILE:	DiffCon.c
DESCRIPTION:	Housekeeping and command parsing functions.
END DESCRIPTION ************************************************/


/********* Function PROTOTYPES (follow syntax, including comment lines) ***************************/
/*** BeginHeader CommandHandler, etc */
void CommandHandler (char* Cmd, int InLen);
/*** EndHeader ***********************/

#ifndef DO_DEBUG
	#nodebug								// disables insertion of breakpoints for debugging, this file only.
#endif


/*************************** Incoming Command From Host *******************************************/
void CommandHandler(char* Cmd, int InLen)  {
/* START FUNCTION DESCRIPTION ******************************************
CommandHandler    <ChromaMatic2 Commands.c>
DESCRIPTION:	Some command data has arrived from host.  Process it.
PARAMETER 1:	char* Cmd, incoming data, will be processed.
PARAMETER 2:   int Len, the length of the command and packet.
COMMENTS:		Assumed to have one command per packet. (New to Rabbit Command Handler)
END DESCRIPTION *******************************************************/
	char PllLock[1], C[10];												//a channel data to be sent to the FPGA.
	int  PhiDel, Dat;
	float Period, Val;

	switch (Cmd[0])  {
		case 'H':																  //ECHO INCOMING HEARTBEAT (1 byte).
			if (InLen == 1)																				//valid command.
				udp_send(&HostSock, Cmd, 1);													 //echo the heartbeat.
			break;

		case 'D':																			 //SET DC VOLTAGE (7 bytes).
			if (InLen == 7)  {																			//valid command.
				strncpy (Settings+1, Cmd, 7);											 //update the settings list.
				strncpy (Buf1, Cmd+1, 6);													 //extract the frequency.
				Buf1[6] = 0x0;
				Val = atof(Buf1);													  //convert the string to a float.
				if (Val > 1)   Val = 1;																	  //limit to 1.
				if (Val < -1)  Val = -1;
				Dat = 8192 + (int)(Val * 8192);										//convert to 14-bit bipolar.
				while (Dat > 16383)  Dat--;						  						 //for +1.000 or <-1.000.
				DacSet (Dat);
			}
			break;

		case 'F':																//SET OSCILLATOR FREQUENCY (5 bytes).
			if (InLen == 5)  {																			//valid command.
				strncpy (Settings+9, Cmd, 5);											 //update the settings list.
				strncpy (Buf1, Cmd+1, 4);													 //extract the frequency.
				Buf1[4] = 0x0;
				Freq = atoi(Buf1);
				if (Freq < FREQ_MIN)  Freq = FREQ_MIN;			 					  						//minimum.
				if (Freq > FREQ_MAX)  Freq = FREQ_MAX;	 						  							//maximum.
				FreqSet (Freq);
			}
			break;

		case 'P':													  //SET PHASE ANGLE FOR MEASURMENTS (4 bytes).
			if (InLen == 4)  {																			//valid command.
				strncpy (Settings+15, Cmd, 4);										 //update the settings list.
				strncpy (Buf1, Cmd+1, 3);														  //extract the phase.
				Buf1[3] = 0x0;
				Dat = atoi(Buf1);													  //convert the string to integer.
				if (Dat >= 0 && Dat <= 359)  Phi = (Dat + PHI_OFFSET + 180.0) / 360.0;		 //normalize.
				if (Phi < 0)  Phi += 1.0;												  //too low, use next cycle.
			}
			break;

		case 'A':																			 //SET AC VOLTAGE (4 bytes).
			if (InLen == 4)																				//valid command.
				strncpy (Settings+34, Cmd, 4);  										 //update the settings list.
				strncpy (Buf1, Cmd+1, 3);														  //extract the phase.
				Buf1[3] = 0x0;
				Dat = atoi(Buf1);													  //convert the string to integer.
				if (Dat >= 0 && Dat <= 255)  PotSet (Dat, 0);																 //set the pots.
			break;

		case 'G':															//SET AC VOLTAGE GAIN DECADES (3 bytes).
			if (InLen == 3)  {																			//valid command.
				strncpy (Settings+26, Cmd, 3);										 //update the settings list.
				if (Cmd[1] == '3')  RelayLatch(2);									 //Set AC-V Gain Relay To 3.
				else                RelayLatch(1);									 //set AC-V gain relay to 1.
				switch (Cmd[2])  {																	 //Set Gain Decade.
					case '0':																					  //Gain = 1.
						BitWrPortI (PADR, &PADRShadow, 0, 6);							 		//set A0 (SPI clk).
						BitWrPortI (PADR, &PADRShadow, 0, 7);		 					 	  //set A1 (SPI data).
						break;
					case '1':																					 //Gain = 10.
						BitWrPortI (PADR, &PADRShadow, 1, 6);							 		//set A0 (SPI clk).
						BitWrPortI (PADR, &PADRShadow, 0, 7);		 					 	  //set A1 (SPI data).
						break;
					case '2':																					//Gain = 100.
						BitWrPortI (PADR, &PADRShadow, 0, 6);							 		//set A0 (SPI clk).
						BitWrPortI (PADR, &PADRShadow, 1, 7);		 					 	  //set A1 (SPI data).
						break;
					case '3':																				 //Gain = 1,000.
						BitWrPortI (PADR, &PADRShadow, 1, 6);							 		//set A0 (SPI clk).
						BitWrPortI (PADR, &PADRShadow, 1, 7);		 					 	  //set A1 (SPI data).
						break;
					default:  break;
				}
				BitWrPortI (PADR, &PADRShadow, 0, 1);							 					 //set !AC-V_CS.
				BitWrPortI (PADR, &PADRShadow, 1, 1);							 				  //clear !AC-V_CS.
			}
			break;

		case 'C':															//SET AC CURRENT GAIN DECADES (3 bytes).
			if (InLen == 3)  {																			//valid command.
				strncpy (Settings+30, Cmd, 3);										 //update the settings list.
				if (Cmd[1] == '3')  RelayLatch(8);									 //Set AC-I Gain Relay To 3.
				else                RelayLatch(4);									 //set AC-I gain relay to 1.
				switch (Cmd[2])  {																	 //Set Gain Decade.
					case '0':																					  //Gain = 1.
						BitWrPortI (PADR, &PADRShadow, 0, 6);							 		//set A0 (SPI clk).
						BitWrPortI (PADR, &PADRShadow, 0, 7);		 					 	  //set A1 (SPI data).
						break;
					case '1':																					 //Gain = 10.
						BitWrPortI (PADR, &PADRShadow, 1, 6);							 		//set A0 (SPI clk).
						BitWrPortI (PADR, &PADRShadow, 0, 7);		 					 	  //set A1 (SPI data).
						break;
					case '2':																					//Gain = 100.
						BitWrPortI (PADR, &PADRShadow, 0, 6);							 		//set A0 (SPI clk).
						BitWrPortI (PADR, &PADRShadow, 1, 7);		 					 	  //set A1 (SPI data).
						break;
					case '3':																				 //Gain = 1,000.
						BitWrPortI (PADR, &PADRShadow, 1, 6);							 		//set A0 (SPI clk).
						BitWrPortI (PADR, &PADRShadow, 1, 7);		 					 	  //set A1 (SPI data).
						break;
					default:
						break;
				}
				BitWrPortI (PADR, &PADRShadow, 0, 0);							 					 //set !AC-I_CS.
				BitWrPortI (PADR, &PADRShadow, 1, 0);							 				  //clear !AC-I_CS.
			}
			break;

		case 'Q':																//SET SAMPLES IN AVERAGING (5 bytes).
			if (InLen == 5)  {																			//valid command.
				strncpy (Settings+20, Cmd, 5);										 //update the settings list.
				strncpy (Buf1, Cmd+1, 4);													 //extract the frequency.
				Buf1[4] = 0x0;
				Dat = atoi(Buf1);													  //convert the string to integer.
				if (Dat > 0 && Dat < 10000)  SamplesToAvg = Dat;
			}
			break;

		case 'M':											//READ ALL VOLTAGES AND CURRENTS AT SET PHASE ANGLE.
			if (InLen >= 1)  {																			//valid command.
				Period = TIMER_FREQ / (float)Freq;
				PhiDel = (int)(Period * Phi);												 //Calculate Phase Delay.
			   WrPortI (TCS3HR, NULL, PhiDel / 256);					 //ADC Convert high: Timer C3 set MS.
	   		WrPortI (TCS3LR, NULL, PhiDel % 256);											 //Timer C3 set LS.
	   		PhiDel += TIMER_RST_DELAY;									 //keep going, to prevent retriggers.
	   		WrPortI (TCR3HR, NULL, PhiDel / 256);					//ADC Convert low: Timer C3 reset MS.
	   		WrPortI (TCR3LR, NULL, PhiDel % 256);										  //Timer C3 reset LS.
	   		PhiDel ++;																 //loop back Timer C bit later.
	         PhaseMS  = PhiDel / 256;										//Timer C limit when phase timing.
	         PhaseLS  = PhiDel % 256;
	         AvgCount = SamplesToAvg;											  //initiate averaging counter.
	         DoAdcs   = 1;																				 //request ADCs.
	      }
			break;

		case 'S':																		  //SEND ALL SETTINGS (1 byte).
			if (InLen == 1)  {																			//valid command.
				udp_send(&HostSock, Settings, 48);
				strncpy (Settings+39, "00000000", 8);									//reset saturation flags.
			}
			break;

		default:  break;
	}
}


/**************************************************************************************************/