/* START LIBRARY DESCRIPTION *********************************************
FILE:				DiffCon Hardware.c
PARENT FILE:	DiffCon.c
DESCRIPTION:	Hardware specific functions such as Rabbit
END DESCRIPTION **********************************************************/

#ifndef DO_DEBUG
	#nodebug								// disables insertion of breakpoints for debugging, this file only.
#endif

/*** BeginHeader BoardBoot */
void BoardBoot (void);
void RabbitBoot (void);
void RelayBoot (void);
void RelayLatch (int Rly);
void FreqBoot (void);
void FreqSet (int Fout);
void DacSet (unsigned int Vout);
void PotSet (int A, int B);
void IsrOsc();
void IsrTimerC3();
/*** EndHeader */


/**************************** Boot The Hardware On The Board **************************************/
void BoardBoot (void)  {
/* START FUNCTION DESCRIPTION ****************************
BoardBoot
DESCRIPTION:	Setup both threshold DACs, 24 delays,
               read the FPGA version, turn off all outputs
END DESCRIPTION *****************************************/

	RabbitBoot();
	DisplayBoot();
	FreqBoot();
	FreqSet(1000);
	DacSet (8192);																				 //and set the DAC to 0V.
	PotSet (0, 0);
	CommandHandler("C10", 3);															  //set the AC current gain.
	CommandHandler("G10", 3);															  //set the AC voltage gain.
	RelayBoot();
}


/*********************************** Set the Dac DC Output ****************************************/
void DacSet (unsigned int Vout)  {
/* START FUNCTION DESCRIPTION ********************************************
DacSet   <DiffCon Hardware.c>
DESCRIPTION:	Set the DC output voltage using the AD5552 in bipolar mode.
PARAMETER 1:	Output voltage, 0-16383, corresponding to 4.096V.
END DESCRIPTION *********************************************************/
	int Bit;

	BitWrPortI (PADR, &PADRShadow, 0, 6);								//set SPI clk low (rising edge load).
	BitWrPortI (PADR, &PADRShadow, 0, 4);										 //set DAC !CS low, enable SPI.
	for (Bit = 0x2000;  Bit > 0;  Bit /= 2)  {							 //bang in the 14 bits, MSB first.
		BitWrPortI (PADR, &PADRShadow, (Vout & Bit), 7); 										 //set data bit.
		BitWrPortI (PADR, &PADRShadow, 1, 6);													//set SPI clk high.
		BitWrPortI (PADR, &PADRShadow, 0, 6);													 //set SPI clk low.
	}
	BitWrPortI (PADR, &PADRShadow, 1, 4);									  //set DAC !CS high, disable SPI.
}


/*********************************** Set the Pots for the AC Voltage ******************************/
void PotSet (int A, int B)  {
/* START FUNCTION DESCRIPTION *****************************************
PotSet   <DiffCon Hardware.c>
DESCRIPTION:	Set the AC output voltage using the AD5162 pot.
PARAMETER 1:	Ratiometric pot A, 0 to 255
PARAMETER 2:	Trim pot B, 0 to 255
NOTE:				Error handling is implicit because only 8-LSBs are used.
END DESCRIPTION ******************************************************/
	int Bit;

	A =  A % 256;																		 //CONDITIONING AND ADDRESSING.
	B = (B % 256) + 0x100;

	BitWrPortI (PADR, &PADRShadow, 0, 6);								//set SPI clk low (rising edge load).
																										//POT A (RATIOMETRIC).
	BitWrPortI (PADR, &PADRShadow, 0, 3);										 //set Pot !CS low, enable SPI.
	for (Bit = 0x100;  Bit > 0;  Bit /= 2)  {				 //bang in the 9 bits, address then MSB first.
		BitWrPortI (PADR, &PADRShadow, (A & Bit), 7);	 										 //set data bit.
		BitWrPortI (PADR, &PADRShadow, 1, 6);													//set SPI clk high.
		BitWrPortI (PADR, &PADRShadow, 0, 6);													 //set SPI clk low.
	}
	BitWrPortI (PADR, &PADRShadow, 1, 3);									  //set Pot !CS high, disable SPI.
												 																 //POT B (TRIM).
	BitWrPortI (PADR, &PADRShadow, 0, 3);										 //set Pot !CS low, enable SPI.
	for (Bit = 0x100;  Bit > 0;  Bit /= 2)  {				 //bang in the 9 bits, address then MSB first.
		BitWrPortI (PADR, &PADRShadow, (B & Bit), 7);	 										 //set data bit.
		BitWrPortI (PADR, &PADRShadow, 1, 6);													//set SPI clk high.
		BitWrPortI (PADR, &PADRShadow, 0, 6);													 //set SPI clk low.
	}
	BitWrPortI (PADR, &PADRShadow, 1, 3);									  //set Pot !CS high, disable SPI.
}


/**************************** OSCILLATOR ISR ******************************************************/
nodebug root interrupt void IsrOsc()  {    //Zero crossing at 0 of AC signal.  If an ADC reading is
							  // requested, set the timer (Timer C) to run for longer than the phase delay.
	if (DoAdcs)  {
	   DoAdcs = 0;											  //cancel request for ADCs because implementing it.
	   WrPortI (TCCSR, NULL, 0);                                            //Timer C clock disabled.
	   WrPortI (TCDHR, NULL, PhaseMS);												  //SET TIMER C DIVIDER, MS.
	   WrPortI (TCDLR, NULL, PhaseLS);     	                                 //Timer C divider, LS.
	   WrPortI (TCCSR, NULL, 1);                                             //Timer C clock enabled.
	}
}


/**************************** TIMER C3 ISR ********************************************************/
nodebug root interrupt void IsrTimerC3() {//Output the ADC Convert signal.  Triggered by rising edge
			// of PD1, means that Timer C3 (out on PC3, wired to PD1) has timed out.  Put Timer C on a
			// short loop, effectively reset.
   WrPortI (TCCSR, NULL, 0);                                          		//Timer C clock disabled.
	BitWrPortI (PCDR, &PCDRShadow, 1, 1);												  //set ADC Convert true.
   WrPortI (TCDHR, NULL, 0x00);                      									//Timer C divider, MS.
   WrPortI (TCDLR, NULL, 0x01);                     									//Timer C divider, LS.
   WrPortI (TCCSR, NULL, 1);                                         		 //Timer C clock enabled.
}


/*********************************** Boot the Rabbit **********************************************/
void RabbitBoot (void)  {			//Setup Rabbit RCM4200 I/O ports and registers.
/* START FUNCTION DESCRIPTION ********************
BootRabbit
DESCRIPTION:	Setup all Rabbit RCM4200 registers and ports
END DESCRIPTION *********************************/

	/***  PORT A ***/																	//CHIP SELECTS AND SPI OUTPUTS.
	WrPortI (SPCR, &SPCRShadow, 0x84);															//PA[] all outputs.
	WrPortI (PADR, &PADRShadow, 0x1C);										  //DAC_LDAC low, no CS, SPI idle.

	/***  PORT B ***/																  //ADC INPUTS AND RELAY CONTROLS.
	WrPortI (PBDDR, &PBDDRShadow, PBDDRShadow | 0x60);										  //PB[6:5] output.

	/***  PORT C ***/							 //PC[0] display, PC[1] ADC Convert, PC[3] int from Timer C3.
	WrPortI (PCDDR, &PCDDRShadow, PCDDRShadow | 0x0B);										//PC[3,1:0] output.
	WrPortI (PCFR,  &PCFRShadow,  PCFRShadow  | 0x09);						  //PC[3,0] alternate function.
	WrPortI (PCALR, &PCALRShadow, PCALRShadow | 0x80);			  //PC[0] alt (TXD), PC[3] alt Timer C3.

	/***  PORT D ***/																 //PC[1] Timer C3 input interrupt.
	WrPortI (PDDDR, &PDDDRShadow, 0x00);														  //PC[] all input.

	/***  PORT E ***/																					//Oscillator input.
	WrPortI (PEDDR, &PEDDRShadow, PEDDRShadow & 0xFE);											  //PE[0] input.

	/*** TIMER C ***/														//base timer for oscillator phase delay.
	WrPortI (TCCR,  NULL, 0x08);												//Peripheral Clk/16, no interrupt.
	WrPortI (TCCSR, NULL, 1);																			//clock enabled.

	/*** EXT INTERRUPTS ***/  			 //(PD1 Timer C3 input interrupt, PE0 Oscillator zero crossing).
	SetVectExtern(0, IsrOsc);											  //Oscillator zero crossing ISR vector.
	WrPortI (I0CR, NULL, 0x07);											  //Int0 on falling edge, priority 3.
	SetVectExtern(1, IsrTimerC3);																//Timer C3 ISR vector.
	WrPortI (I1CR, NULL, 0x83);												//Int1 on rising edge, priority 3.
}


/***************** Relay Driver (expletive deleted) Boot And Write Register ***********************/
void RelayRegWr (int Wrd)  {
/* START FUNCTION DESCRIPTION *****************************************
RelayRegWr  <DiffCon Hardware.c>
DESCRIPTION:	Write a command word to a TLE7230 relay driver register.
PARAMETER 1:	The 16-bit word to be written.
NOTES:			Data loads on Clk falling edge, MSB first.
END DESCRIPTION ******************************************************/

	unsigned int Ptr;

	BitWrPortI (PADR, &PADRShadow, 0, 6);											 //ensure SPI clk low [sic].
	BitWrPortI (PBDR, &PBDRShadow, 0, 5);													  //set relay !CS low.
	for (Ptr = 0x8000;  Ptr > 0;  Ptr /= 2)  {							 //bang in the 16 bits, MSB first.
		BitWrPortI (PADR, &PADRShadow, (Wrd & Ptr), 7);											 //set data bit.
		BitWrPortI (PADR, &PADRShadow, 1, 6);													//set SPI clk high.
		BitWrPortI (PADR, &PADRShadow, 0, 6);													 //set SPI clk low.
	}
	BitWrPortI (PBDR, &PBDRShadow, 1, 5);													 //set relay !CS high.
}

void RelayBoot (void)  {
/* START FUNCTION DESCRIPTION *********************
RelayBoot  <DiffCon Hardware.c>
DESCRIPTION:	Cold boot the TLE7230G relay driver
END DESCRIPTION *********************************/
	RelayRegWr (0x8000 + 0x000 + 0x00);														//reset all registers.
	RelayRegWr (0xC000 + 0x100 + 0x00);																	  //set ICMR.
	RelayRegWr (0xC000 + 0x200 + 0x00);																	  //set BOCR.
	RelayRegWr (0xC000 + 0x300 + 0x00);																	  //set OLCR.
	RelayRegWr (0xC000 + 0x400 + 0x00);																	  //set OTCR.
	RelayRegWr (0xC000 + 0x500 + 0xFF);																	  //set SRCR.
	RelayRegWr (0xC000 + 0x700 + 0x00);																	  //set CTL.
	RelayLatch (0xA);  Pause (50);						//switch relays back and forth to clean contacts.
	RelayLatch (0x5);  Pause (50);
	RelayLatch (0xA);  Pause (50);
	RelayLatch (0x5);  Pause (50);
	RelayLatch (0xA);  Pause (50);
	RelayLatch (0x5);														  //set AC-V and AC-I relays to gain x1.
}


/********************* Latch A Relay On Or Off ****************************************************/
void RelayLatch (int Rly)  {
/* START FUNCTION DESCRIPTION **************************
RelayLatch   <DiffCon Hardware.c>
DESCRIPTION:  Blip the requested relay coil to latch it.
PARAMETER 1:  Number of the channel, 0 to 7.
NB:  RelayRegWr() does not have a prototype so it must
     appear before RelayLatch().
END DESCRIPTION ***************************************/

	BitWrPortI (PBDR, &PBDRShadow, 0, 6);													  //turn on relay +5V.
	RelayRegWr (0xC700 + Rly);																				   //set CTL.
	Pause (10);
	BitWrPortI (PBDR, &PBDRShadow, 1, 6);													 //turn off relay +5V.
}


/**************************** Boot the Oscillator *************************************************/
void FreqBoot (void)  {
/* START FUNCTION DESCRIPTION ****************
FreqBoot
DESCRIPTION:	Cold boot the oscillator.
END DESCRIPTION *****************************/

	unsigned int Ptr, Dat;

	Dat = 0x2100;		//word for reset															//RESET OSCILLATOR.
	BitWrPortI (PADR, &PADRShadow, 1, 6);							 //set SPI clk high (falling edge load).
	BitWrPortI (PADR, &PADRShadow, 0, 2);										 //set osc !CS low, enable SPI.
	for (Ptr = 0x8000;  Ptr > 0;  Ptr /= 2)  {							 //bang in the 16 bits, MSB first.
		BitWrPortI (PADR, &PADRShadow, (Dat & Ptr), 7);											 //set data bit.
		BitWrPortI (PADR, &PADRShadow, 0, 6);													 //set SPI clk low.
		BitWrPortI (PADR, &PADRShadow, 1, 6);													//set SPI clk high.
	}
	BitWrPortI (PADR, &PADRShadow, 1, 2);									  //set osc !CS high, disable SPI.

	Dat = 0x2000;		//word for initial setup, no reset									//SETUP OSCILLATOR.
	BitWrPortI (PADR, &PADRShadow, 1, 6);							 //set SPI clk high (falling edge load).
	BitWrPortI (PADR, &PADRShadow, 0, 2);										 //set osc !CS low, enable SPI.
	for (Ptr = 0x8000;  Ptr > 0;  Ptr /= 2)  {							 //bang in the 16 bits, MSB first.
		BitWrPortI (PADR, &PADRShadow, (Dat & Ptr), 7);											 //set data bit.
		BitWrPortI (PADR, &PADRShadow, 0, 6);													 //set SPI clk low.
		BitWrPortI (PADR, &PADRShadow, 1, 6);													//set SPI clk high.
	}
	BitWrPortI (PADR, &PADRShadow, 1, 2);									  //set osc !CS high, disable SPI.
}


/**************************** Set the Oscillator Frequency ****************************************/
void FreqSet (int Fout)  {
/* START FUNCTION DESCRIPTION ****************
FreqSet
DESCRIPTION:	Setup the oscillator frequency.
PARAMETER 1:	Frequency in Hz, 25 to 1000Hz.
END DESCRIPTION *****************************/

	unsigned int Bit, FreqReg;
	float fScale;

	fScale = Fout * 26.8435;	 //where f = Clk/2^28 * FreqReg, Clk = 10MHz and 26.8435 = 2^28 / Clk.
	FreqReg = 0x4000 + ((unsigned int)(fScale) % 16384);	  							//SEND LS 14-BIT WORD.
	BitWrPortI (PADR, &PADRShadow, 1, 6);							 //set SPI clk high (falling edge load).
	BitWrPortI (PADR, &PADRShadow, 0, 2);										 //set osc !CS low, enable SPI.
	for (Bit = 0x8000;  Bit > 0;  Bit /= 2)  {							 //bang in the 16 bits, MSB first.
		BitWrPortI (PADR, &PADRShadow, (FreqReg & Bit), 7); 									 //set data bit.
		BitWrPortI (PADR, &PADRShadow, 0, 6);													 //set SPI clk low.
		BitWrPortI (PADR, &PADRShadow, 1, 6);													//set SPI clk high.
	}

	FreqReg = 0x4000 + ((unsigned int)(fScale) / 16384); 								//SEND MS 14-BIT WORD.
	BitWrPortI (PADR, &PADRShadow, 0, 2);										 //set osc !CS low, enable SPI.
	for (Bit = 0x8000;  Bit > 0;  Bit /= 2)  {							 //bang in the 16 bits, MSB first.
		BitWrPortI (PADR, &PADRShadow, (FreqReg & Bit), 7);	 								 //set data bit.
		BitWrPortI (PADR, &PADRShadow, 0, 6);													 //set SPI clk low.
		BitWrPortI (PADR, &PADRShadow, 1, 6);													//set SPI clk high.
	}
	BitWrPortI (PADR, &PADRShadow, 1, 2);									  //set osc !CS high, disable SPI.
}



/********************* Check If ADCs Are Converting And Get All Data If Ready *********************/
void AdcHandler (void)  {
/* START FUNCTION DESCRIPTION ****************************************
Adc Handler   <DiffConn Hardware.c>
DESCRIPTION:	If the ADCs have done a conversion, read the four ADCs,
               sum each for averaging. If still have more samples to
               average, request ADCs again. Repeat numer of times for
               averaging, then send them to the host in a packet.
END DESCRIPTION *****************************************************/
	unsigned long Bit, Vdc, Vac, Idc, Iac;
	static unsigned long VdcSum, VacSum, IdcSum, IacSum;

	if (BitRdPortI (PCDR, 1))  {						//ADCs conversion requested.	 //ADC VALUES TO HOST.
		if (AvgCount == SamplesToAvg)  {											  //start of new averaging run.
		   VdcSum = 0;  VacSum = 0;  IdcSum = 0;  IacSum = 0;  }
	   Vdc = 0;  Vac = 0;  Idc = 0;  Iac = 0;
		BitWrPortI (PADR, &PADRShadow, 0, 6);													 //set SPI clk low.
	   BitWrPortI (PCDR, &PCDRShadow, 0, 1);										  //set the ADC Convert low.

		for (Bit = 0x8000;  Bit > 0;  Bit >>= 1)  {	 		 //READ THE 16-BITS, MSB FIRST, ADD TO SUM.
			if (BitRdPortI (PBDR, 1))  Vdc += Bit;														 //read DC-V.
			if (BitRdPortI (PBDR, 2))  Vac += Bit;														 //read AC-V.
			if (BitRdPortI (PBDR, 3))  Idc += Bit;														 //read DC-I.
			if (BitRdPortI (PBDR, 4))  Iac += Bit;														 //read AC-I.
			BitWrPortI (PADR, &PADRShadow, 1, 6);												//set SPI clk high.
			BitWrPortI (PADR, &PADRShadow, 0, 6);												 //set SPI clk low.
		}
		VdcSum += Vdc;																						  //Add To Sums.
		VacSum += Vac;
		IdcSum += Idc;
		IacSum += Iac;
		if (Vdc == 0x0000)  Settings[39] = '1';										  //Set Saturation Flags.
		if (Vdc == 0xFFFF)  Settings[40] = '1';
		if (Vac == 0x0000)  Settings[41] = '1';
		if (Vac == 0xFFFF)  Settings[42] = '1';
		if (Idc == 0x0000)  Settings[43] = '1';
		if (Idc == 0xFFFF)  Settings[44] = '1';
		if (Iac == 0x0000)  Settings[45] = '1';
		if (Iac == 0xFFFF)  Settings[46] = '1';
		if (--AvgCount > 0)  DoAdcs = 1;								  //still averaging, request ADCs again.

		else  {												  //FINISHED AVERAGING, SEND THE PACKET TO THE HOST.
	      Buf1[0] = 'D';
	      ltoa (VdcSum/SamplesToAvg, Buf1+1);   //DC-V: append the null terminated value as a string.
	      while (strlen(Buf1) < 6)  strcat (Buf1, " ");                //pad to proper string length.

	      ltoa (VacSum/SamplesToAvg, Buf1+6);   //AC-V: append the null terminated value as a string.
	      while (strlen(Buf1) < 11)  strcat (Buf1, " ");               //pad to proper string length.

	      ltoa (IdcSum/SamplesToAvg, Buf1+11);  //DC-I: append the null terminated value as a string.
	      while (strlen(Buf1) < 16)  strcat (Buf1, " ");               //pad to proper string length.

	      ltoa (IacSum/SamplesToAvg, Buf1+16);  //AC-I: append the null terminated value as a string.
	      while (strlen(Buf1) < 21)  strcat (Buf1, " ");               //pad to proper string length.

	      udp_send(&HostSock, Buf1, strlen(Buf1));                        //send the ADC data packet.
	   }
	}
}


/*********************************** Help descriptions from Pulsar.c ******************************/

/* START FUNCTION DESCRIPTION *******************************
Pause
DESCRIPTION:	Stop and wait "mS" interval before leaving this function.
PARAMETER 1:	int mS; time in mS, from 1 to 32,767.
END DESCRIPTION ********************************************/

/* START FUNCTION DESCRIPTION *******************************
ShowSysInfo
DESCRIPTION:	Usually in debug mode, show system information on stdio.
END DESCRIPTION ********************************************/

/**************************************************************************************************/

