/***************************************************************************************************
FILE: DiffCon.c
INITIAL CODING OF THIS PROJECT: Started May 2012, Alan Stummer, astummer-at-physics.utoronto.ca
REVISIONS:  None, yet.
COMPILER/IDE:  Dynamic C 10.56
PLATFORM:  RCM4200.
PROJECT OVERVIEW FROM ELECTRICAL PERSPECTIVE:
   The Pulsar generates pulses based on timing of a pulse laser.  There are 20 output channels, each
   one is independant.  The realtime pulses come from the FPGA.  The host sends timing requirements
   to the Rabbit which transfers it to registers in the FPGA.  Upon a Run Enable signal and a
   trigger signal, the FPGA pulses the outputs according to the registered timing.
RABBIT'S ROLE IN THE PROJECT:
The Rabbit is between the host computer and the DDS.  The Rabbit:
  - receives operation commands and parameters from the UDP host.
  - sets all operation parameters in the FPGA.
  - send status reports to the UDP monitor Warren.
DESCRIPTION OF ADC AQUISITION
 1) The realtime timer (Timer C) is normally set to a short sub-uS loop.
 2) Host sets the oscillator frequency.
 3) Host sends "9" or "2" command to request ADCs at 90 or 270.  The MS and LS phase delay timing
    is calculated (to save time in the ISR).  DoAdcs is set.
 5) The zero crossing of the ocsillator always runs the IsrOsc ISR.
   5a) If in that ISR DoAdcs is set, the real time timer (Timer C) is allowed to run long and the
       phase delay timer (Timer C3) is set. The phase delay timer is output on PC3 and input back in
       on PD1, with PD1 triggering the IsrPhaseDelay ISR.
 7) After the phase delay (Timer C3 trips), the ADCs conversion is started, the realtime timer
    (Timer C) is put back on a short loop and the phase delay timer (Timer C3) is set long so that
    it never trips.
 8) The main loop polls the ADC Convert signal.  If true, it assumes the conversion is over (takes
    3.2uS), reads the ADCs and sends them to the host.
***************************************************************************************************/

//Project specific constants
#define DO_DEBUG  	  //normally commented out, otherwise will allow debugging through serial port.
#define _PRIMARY_STATIC_IP  "192.168.1.161" 	//normally commented out, use to run in Alan's room.
#define VERSION			"0.0.4"						 //appears on the monitor and display, max 10 chars.

#define HB_TIMEOUT		2500			//max time between incoming heartbeats before loss of host (mS).
#define MONITOR_PORT		6595										 // Port for UDP socket server for monitor.
#define FIRST_HOST_ONLY	0				 //after boot, only the first host to connect can send commands.
#define ANY_HOST			1											  //after boot, any host can send commands.
#define TIMER_FREQ      1843200						 //clock frequency of Timer C (to time phase delay).
#define TIMER_RST_DELAY 100			 //count to allow Timer C reset, will double pulse if too short.
#define PHI90           0.22						  //normalized part of oscillator cycle for 90 sample.
#define PHI270          0.71						 //normalized part of oscillator cycle for 270 sample.
#define FREQ_MIN        25													 //minimum oscillator frequency (Hz).
#define FREQ_MAX        1000												 //maximum oscillator frequency (Hz).
// Display parameters for brightness and contrast are in the display file, unless defined here.

//Project files  (remember to duplicate this list in "Pulsar Library List.dir")
#use "DiffCon Hardware.c"												  //project specific hardware functions.
#use "Unit Identification.h"																 //unit specific defines.
#use "DiffCon Commands.c"													//project specific command functions.
#use "Display NHD-0220D3Z.c"					//Newhaven Displays NHD-0220D3Z-xxx-xxx display functions.

//Dynamic C directives and required defines
#class auto
#define STDIO_DISABLE_FLOATS												//disable floating point for std I/O.
#define TCPCONFIG           1	  		 		  //static IP.  Required only to eliminate compile errors.
#define MY_GATEWAY          "10.10.6.1"			  //don't need but gives compiler warning otherwise.
#define MY_NAMESERVER       "10.10.6.1"			  //don't need but gives compiler warning otherwise.
#define MAX_UDP_SOCKET_BUFFERS	2			  // max number of UDP sockets that can be opened at once.
#define UDP_BUF_SIZE 9000								  //to accomodate the counters Tx of 8 x 1026 bytes.
#define DINBUFSIZE  31							 //for nonexistant serial port, to stop compiler warnings.
#define DOUTBUFSIZE 31							 //for nonexistant serial port, to stop compiler warnings.
#define COUTBUFSIZE 127									  				  						//serial port buffers.
#define CINBUFSIZE  127

//Dynamic C library fies
#use "dcrtcp.lib"																			 //Rabbit's TCP/UDP library.

//Precomiler directives
#ifndef DO_DEBUG
	#nodebug								// disables insertion of breakpoints for debugging, this file only.
#endif

/*** BeginHeader Pause*/
nodebug void Pause (int mS);
void ShowSysInfo (void);
void SendVersionsToHost (void);
void AdcHandler (void);
//Global variables
udp_Socket HostSock;
udp_Socket MonitorSock;
char RabbitVersion[11], FpgaVersion[11];								//formatted string, e.g. "2.199.0\0".
char Buf1[70], Buf2[70]; 												//general use buffers, use with caution.
int  DoAdcs;												  //flag to say to read ADCs on next oscillator ISR.
int  Saturation;																				  //ADC saturation flags.
int  Freq;																					//oscillator frequency (Hz).
int  SamplesToAvg, AvgCount;					//number of samples to average, current number of samples.
int  PhaseMS, PhaseLS;							  //phase delay time, precalculated to reduce time in ISR.
/*** EndHeader */


/************************************** main() ****************************************************/
void main()  {		/*Cold boot, then loop forever, always handling host communications.*/
	int ConnectedToHost, InLen;
	unsigned long LastIncomingHb, LastSec;
	char Incoming[100];

	if (strncmp(_PRIMARY_STATIC_IP, "192.", 4) == 0)  {
		#warnt "WARNING: Using Alan's office IP address!"
	}

	strcpy (RabbitVersion, VERSION);
	#ifdef DO_DEBUG
		strcat (RabbitVersion, "dbg");
	#endif

	BoardBoot();
	DisplayWrite (UNIT_NAME, "Booting...", BRIGHT);
	Pause (2000);

	sprintf (Buf1, "Rabbit: %s", RabbitVersion);
	DisplayWrite (Buf1, " ", BRIGHT);
	Pause (2000);

	DisplayWrite (UNIT_NAME, "Starting phy...", BRIGHT);									//START PHY DEVICE.
	sock_init();										 //following can use either: FIRST_HOST_ONLY, ANY_HOST.
	if (!udp_open(&HostSock, HOST_PORT, FIRST_HOST_ONLY, HOST_PORT, NULL));	  //OPEN UDP HOST SOCKET.
  	else  while(!sock_established(&HostSock))  if (!tcp_tick(&HostSock))  break;

	sprintf(Buf1, "%s", _PRIMARY_STATIC_IP);	  													 //SHOW IP INFO.
	sprintf(Buf2, "UDP Port %u", HOST_PORT);
	DisplayWrite (Buf1, Buf2, BRIGHT);
																					  //OPEN A UDP SOCKET TO THE MONITOR.
	if (!udp_open(&MonitorSock, 0, resolve(MONITOR_ADDRESS), MONITOR_PORT, NULL));	  //open socket.
  	else  {
		while(!sock_established(&MonitorSock))							 				 // establish the socket.
  			if (!tcp_tick(&MonitorSock))  break;
	}
	Pause (2000);

	DisplayWrite (UNIT_NAME, "Waiting for host...", DIM);										 //HOUSEKEEPING.
	sprintf(Buf1, "%s is starting on %s, port %u.\n", UNIT_NAME, _PRIMARY_STATIC_IP, HOST_PORT);
	udp_send(&MonitorSock, Buf1, strlen(Buf1));
	#ifdef DO_DEBUG
		ShowSysInfo();
	   #warnt "FINAL RELEASE WARNING: Running with debugging turned on!"
		sprintf(Buf1, "Warning: %s is in debug mode.\n", UNIT_NAME);
		udp_send(&MonitorSock, Buf1, strlen(Buf1));
	#endif
	LastIncomingHb  = 0;																						  //DEFAULTS.
	ConnectedToHost = 0;
	DoAdcs          = 0;
	Freq            = 1000;
	SamplesToAvg    = 1;
	Saturation      = 0;
	Pause (2000);


		 	/***************************** Main Loop **********************************/
	while(1) {
		AdcHandler();								//after and ADC sample is requested, check if it is ready.

		if (LastIncomingHb + HB_TIMEOUT > MS_TIMER)  { 				 //HAVE HEARTBEAT, CONNECTED TO HOST.
			if (!ConnectedToHost)  {															//First New Heartbeat.
				ConnectedToHost = 1;
				sprintf (Buf2, "%s", inet_ntoa (Buf1, HostSock.hisaddr));
		 		DisplayWrite ("Ready, host is", Buf2, BRIGHT);
		 		SendVersionsToHost();
			}
		}

		else  {																					 //NO HEARTBEAT, NO HOST.
			BitWrPortI (PBDR, &PBDRShadow, 0, 6);													  //set Run low.
			if (ConnectedToHost)  {															//First Missed Heartbeat.
				ConnectedToHost = 0;
				DacSet (0);																			//kill the DC voltage.
				inet_ntoa (Buf1, HostSock.hisaddr);										 //get host's IP address.
				sprintf (Buf2, "%s", Buf1);
				if (HostSock.hisaddr)  DisplayWrite ("Lost contact with", Buf2, DIM);
				else                   DisplayWrite (UNIT_NAME, "Waiting for host...", DIM);

				sock_close (&HostSock);		  //Close Then Reopen UDP Host Socket (not sure why have to).
				if (!udp_open(&HostSock, HOST_PORT, FIRST_HOST_ONLY, HOST_PORT, NULL));
			  	else  while(!sock_established(&HostSock))  if (!tcp_tick(&HostSock))  break;
			}
		}

		tcp_tick (NULL);												//required periodic Rabbit UDP maintenance.

		InLen = udp_recv (&HostSock, Incoming, 99);								 //HANDLE INCOMING COMMANDS.
		if (InLen > 0)  {
			CommandHandler (Incoming, InLen);									   //process command from host.
			LastIncomingHb = MS_TIMER;	 									 //time of last command or heartbeat.
		}
	}
}



/********************* Send Rabbit and FPGA Versions Strings to Host ******************************/
void SendVersionsToHost (void)  {		//Send Rabbit and FPGA version strings to host.
/* START FUNCTION DESCRIPTION ******************************
SendVersionsToHost
DESCRIPTION:   Send Rabbit and FPGA version strings to host.
END DESCRIPTION *******************************************/
	Buf1[0] = 'V';
	Buf1[1] = 0x00;
	strcat (Buf1, RabbitVersion);
	strcat (Buf1, "\n");
	strcat (Buf1, UNIT_NAME);
	udp_send(&HostSock, Buf1, strlen(Buf1));
}



/****************************************** Pause *************************************************/
nodebug void Pause (int mS)  {     //Wait given mS then leave.  Help header is in Pulsar Hardware.c.
	unsigned long Paws;

	Paws = MS_TIMER;
	while (Paws + mS > MS_TIMER);
}


/******************************* Show System Information ******************************************/
void ShowSysInfo (void)  {		//Show some system info on stdio.  Help is in Pulsar Hardware.c.
//	if (SysIDBlock.cpuID)                printf("CPU ID: %d\n", SysIDBlock.CpuID);
	if (SysIDBlock.productID)            printf("Product ID: %d\n", SysIDBlock.productID);
	if (strlen(SysIDBlock.productName))  printf("Product   : %s\n", SysIDBlock.productName);
	if (strlen(SysIDBlock.serialNumber)) printf("Serial #  : %s\n", SysIDBlock.serialNumber);
	if (SysIDBlock.crystalFreq)          printf("Xtal      : %dKHz\n", SysIDBlock.crystalFreq/1000);
	                                     printf("MAC Addr  : %02X:%02X:%02X:%02X:%02X:%02X\n\n",
		SysIDBlock.macAddr[0], SysIDBlock.macAddr[1], SysIDBlock.macAddr[2],
		SysIDBlock.macAddr[3], SysIDBlock.macAddr[4], SysIDBlock.macAddr[5]);
}

/**************************************************************************************************/

